from .agent import IOpipeCore

from .contrib.eventinfo import EventInfoPlugin
from .contrib.profiler import ProfilerPlugin
from .contrib.trace import TracePlugin


class IOpipe(IOpipeCore):
    """
    The default IOpipe agent, pre-loaded with all the bundled plugins.
    """

    def __init__(self, *args, **kwargs):
        configured_plugins = kwargs.pop("plugins", None)
        plugins = [EventInfoPlugin(), ProfilerPlugin(), TracePlugin()]
        if configured_plugins is not None and isinstance(plugins, list):
            plugins = plugins + configured_plugins
        kwargs["plugins"] = plugins

        super(IOpipe, self).__init__(*args, **kwargs)



import json
import logging
import time


class JSONFormatter(logging.Formatter):
    converter = time.gmtime
    default_time_format = "%Y-%m-%d %H:%M:%S"
    default_msec_format = "%s.%03d"

    def formatTime(self, record, datefmt=None):
        """Override to make Python 2/3 compatible"""
        ct = self.converter(record.created)
        t = time.strftime(self.default_time_format, ct)
        s = self.default_msec_format % (t, record.msecs)
        return s

    def format(self, record):
        record.asctime = self.formatTime(record)
        record.message = record.getMessage()
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            if record.message[-1:] != "\n":
                record.message = record.message + "\n"
            record.message = record.message + record.exc_text
        if hasattr(record, "stack_info") and record.stack_info:
            if record.message[-1:] != "\n":
                record.message = record.message + "\n"
            record.message = record.message + self.formatStack(record.stack_info)
        return json.dumps(
            {
                "message": record.message,
                "name": record.name,
                "severity": record.levelname,
                "timestamp": record.asctime,
            },
            sort_keys=True,
        )



import os

from iopipe import IOpipe

iopipe = IOpipe()
wrapped_handler = None


@iopipe
def wrapper(event, context):
    return get_wrapped_handler()(event, context)


def get_wrapped_handler():
    global wrapped_handler

    if not wrapped_handler:
        if "IOPIPE_HANDLER" not in os.environ or not os.environ["IOPIPE_HANDLER"]:
            raise ValueError(
                "No value specified in IOPIPE_HANDLER environment variable"
            )

        try:
            module_path, handler_name = os.environ["IOPIPE_HANDLER"].rsplit(".", 1)
        except ValueError:
            raise ValueError(
                "Improperly formated handler value: %s" % os.environ["IOPIPE_HANDLER"]
            )

        module_path = module_path.replace("/", ".")

        try:
            module = __import__(module_path)
        except ImportError:
            raise ImportError("Failed to import module: %s" % module_path)

        try:
            wrapped_handler = getattr(module, handler_name)
        except AttributeError:
            raise AttributeError(
                "No handler %s in module %s" % (handler_name, module_path)
            )

    return wrapped_handler



class LogWrapper(object):
    def __init__(self, logger, context):
        self.logger = logger
        self.context = context

    def __call__(self, key, value):
        self.context.iopipe.metric(key, value)

    def __getattr__(self, name):
        self.context.iopipe.label("@iopipe/plugin-logger")
        return getattr(self.logger, name)



import abc


def get_plugin_meta(plugins):
    """
    Returns meta data about plugins.

    :param plugins: A list of plugins.
    :type plugins: list
    :returns: A list of dicts containing plugin meta data.
    :rtype: list
    """
    return [
        {
            "name": p.name,
            "version": p.version,
            "homepage": p.homepage,
            "enabled": p.enabled,
        }
        for p in plugins
        if is_plugin(p)
    ]


def is_plugin(plugin):
    """
    Returns true if the plugin implements the `Plugin` interface.

    :param plugin: The plugin to check.
    :returns: True if plugin, False otherwise.
    :rtype: bool
    """
    try:
        return isinstance(plugin, Plugin) or issubclass(plugin, Plugin)
    except TypeError:
        return False


def with_metaclass(meta, *bases):
    """Python 2 and 3 compatible way to do meta classes"""

    class metaclass(meta):
        def __new__(cls, name, this_bases, d):
            return meta(name, bases, d)

    return type.__new__(metaclass, "temporary_class", (), {})


class Plugin(with_metaclass(abc.ABCMeta, object)):
    @abc.abstractproperty
    def name(self):
        return NotImplemented

    @abc.abstractproperty
    def version(self):
        return NotImplemented

    @abc.abstractproperty
    def homepage(self):
        return NotImplemented

    @abc.abstractproperty
    def enabled(self):
        return NotImplemented

    @abc.abstractmethod
    def pre_setup(self, iopipe):
        return NotImplemented

    @abc.abstractmethod
    def post_setup(self, iopipe):
        return NotImplemented

    @abc.abstractmethod
    def pre_invoke(self, event, context):
        return NotImplemented

    @abc.abstractmethod
    def post_invoke(self, event, context):
        return NotImplemented

    @abc.abstractmethod
    def pre_report(self, report):
        return NotImplemented

    @abc.abstractmethod
    def post_report(self, report):
        return NotImplemented
